c--------------------------------------------------------------------------
c	a.bijaoui 85/1 Observatoire de Nice
c	Saidi8 allows to extract fields parameters using an image
c	segmentation by density thresholding
c	Elliptical parameters of the contour
	Subroutine SaiDi8(ImageIn, Ncolumns, Nlines, Threshold,  Label1, 
	1 Label2, Nfieldsmax, LabelInt, LabelFin, Labelflag, NfieldsInt,
	2 Nfields, Npixels, Xcenter, Ycenter, MajorAxis, MinorAxis,
	3 Orientation, Ia, Sx, Sy, Sx2, Sy2, Sxy)
c	ImageIn         = Image Initiale a segmenter
c	Ncolumns        = Nombre de colonnes de cette image
c	Nlines          = Nombre de lignes de cette image
c	Threshold	= Seuil de flux pour la segmentation
c	Label1		= labels of a line
c	Label2		= labels for the following line
c	Nfieldsmax	= Maximum number of fields
c	LabelInt	= Tableau 1d contenant pour chaque domaine l'etiquette
c			  Intermediaire associee
c	LabelFin	= Tableau 1d contenant pour chaque domaine l'etiquette
c			  Finale associee
c	LabelFlag	= array to determine if a field is closed
c	NfieldsInt	= Dimension maximum des Tableaux LabelInt,LabelFin
c		is Ncolumns/2 
c	Nfields         = Nombre de domaines etiquettes
c	Npixels		= Number of pixels in a field
c	Xcenter		= Corresponding central abcissae
c	Ycenter		= Corresponding central ordinate
c	MajorAxis	= ------------- Major Axis
c	MinorAxis	= ------------- Minor Axis
c	Orientation	= ------------- Major axis orientation
c	Ia		= Intermediate array for the area
c	Sx		= -------------------------- xcenter
c	Sy		= -------------------------- ycenter
c	Sx2		= ------------------
c	Sy2		= ------------------
c	Sxy		= ------------------
	Real*4 ImageIn(Ncolumns,Nlines), XCenter(Nfieldsmax),
	1 Ycenter(Nfieldsmax), MajorAxis(NFieldsMax), MinorAxis(Nfieldsmax),
	2 Orientation(NfieldsMax), Sx(NfieldsInt), Sy(NfieldsInt),
	3 Sx2(NfieldsInt), Sy2(NfieldsInt), Sxy(NfieldsInt)
	Integer*4 LabelInt(NfieldsInt), LabelFin(NfieldsInt),
	1 Labelflag(NfieldsInt), Ia(NfieldsInt), Npixels(NfieldsMax),
	2 Label1(Ncolumns), Label2(Ncolumns)
	Nfieldsi=0
	Nfields=0
	Do icolumn=1,Ncolumns
	  Label1(icolumn)=0
	Enddo
	Do iline=1,Nlines
	  Do icolumn=1,Ncolumns
	    if(ImageIn(icolumn,Iline).lt.threshold) then
	      Label2(Icolumn)=0
	    else
	      If(Icolumn.ne.1) then
	        l1=Label2(icolumn-1)
		If(l1.ne.0) Label2(icolumn)=l1
	      endif
	      if(iline.ne.1) then
		l2=Label1(icolumn)
		if(l1.eq.0) then
		  if(l2.eq.0) then
		     Nfieldsi=Nfieldsi+1
		     LabelInt(Nfieldsi)=Nfieldsi
		     Ia(Nfieldsi)=0
		     sx(Nfieldsi)=0.
		     sy(Nfieldsi)=0.
		     sx2(Nfieldsi)=0.
		     sy2(Nfieldsi)=0.
		     sxy(Nfieldsi)=0.
		     Label2(icolumn)=Nfieldsi
	          else
	             Label2(icolumn)=l2
		  endif
		else
		  if(l2.ne.0) then
		    if(l2.ne.l1) then
		      call descen(LabelInt,l1,la1)
		      call descen(LabelInt,l2,la2)
		      la=min0(la1,la2)
		      LabelInt(la1)=la
		      LabelInt(la2)=la
		      Label2(icolumn)=la
		    endif
		  endif
		endif		   
	      endif
	      l2=Label2(icolumn)
	      Ia(l2)=Ia(l2)+1
	      Sx(l2)=Sx(l2)+icolumn
	      Sy(l2)=Sy(l2)+iline
	      Sx2(l2)=Sx2(l2)+Icolumn*Icolumn
	      Sy2(l2)=Sy2(l2)+Iline*Iline
	      Sxy(l2)=Sxy(l2)+Iline*Icolumn
	    endif
	  enddo
	  if(nfieldsi.ne.0) then
	  do ifield=1,Nfieldsi
	    Labelflag(ifield)=0
	    Call descen(LabelInt,Ifield,Jfield)
	    If(Ifield.ne.Jfield) then
	      Sx(jfield)=Sx(jfield)+sx(Ifield)
	      Sy(jfield)=Sy(jfield)+Sy(Ifield)
	      Ia(jfield)=Ia(jfield)+Ia(ifield)
	      Sx2(jfield)=Sx2(jfield)+sx2(Ifield)
	      Sy2(jfield)=Sy2(jfield)+sy2(Ifield)
	      Sxy(jfield)=Sxy(jfield)+sxy(Ifield)
	    endif
	  enddo
	  do icolumn=1,ncolumns
	    l2=label2(icolumn)
	    if(l2.ne.0) then
	      Call Descen(LabelInt,l2,Jfield)
	      LabelFlag(Jfield)=1
	    endif
	  enddo
	  Nfieldso=0
	  Do Ifieldi=1,Nfieldsi
	    If(Ifieldi.eq.LabelInt(Ifieldi)) then
	      If(LabelFlag(Ifieldi).ne.1.or.Iline.eq.Nlines) then
		Nfields=Nfields+1
		Xcenter(Nfields)=Sx(Ifieldi)/Ia(Ifieldi)
		Ycenter(Nfields)=Sy(Ifieldi)/Ia(Ifieldi)
		Npixels(Nfields)=Ia(Ifieldi)
		X1=Xcenter(Nfields)
		Y1=Ycenter(Nfields)
		X2=Sx2(Ifieldi)-x1*x1*Npixels(Nfields)
		Y2=SY2(Ifieldi)-Y1*y1*Npixels(Nfields)
		XY=SXY(Ifieldi)-x1*y1*Npixels(Nfields)
	If(x2.ne.y2) then
	  Orient=Atan(2.*XY/(Y2-X2))/2.
	else
	  Orient=0.7854
	endif
	S=Sin(Orient)
	C=Cos(Orient)
	X=C*C*X2-2.*S*C*XY+S*S*Y2
	Y2=S*S*X2+2.*S*C*XY+C*C*Y2
	X2=x
	if(y2.ne.0..and.x2.ne.0.) then
	el=Sqrt(X2/Y2)
	ss=2.*sqrt(3.14159*sqrt(x2*y2))
	Bx=Sqrt((2./El)*sqrt(sqrt(X2*Y2)/3.14159))
	Ax=Bx*El
	If(el.lt.1.) then
	  B=Ax
	  Ax=Bx
	  Bx=B
	  Orient=Orient+1.57079
	endif
		Orientation(Nfields)=180.*Orient/3.14159
		MajorAxis(Nfields)=Ax
		MinorAxis(Nfields)=Bx
	else
		Nfields=Nfields-1
	endif
		if(Nfields.eq.NfieldsMax) then
		  Type 100
100	format(' You too much output fields - Reserve more place')
		Return
		endif
	      else
		Nfieldso=Nfieldso+1
		LabelFin(Ifieldi)=Nfieldso
		Ia(Nfieldso)=Ia(Ifieldi)
		Sx(Nfieldso)=Sx(Ifieldi)
		Sy(Nfieldso)=Sy(Ifieldi)
		Sx2(Nfieldso)=Sx2(Ifieldi)
		Sy2(Nfieldso)=Sy2(Ifieldi)
		Sxy(Nfieldso)=Sxy(Ifieldi)
	      endif
	    endif
	  enddo
	  endif
	  do icolumn=1,Ncolumns
	    l2=Label2(icolumn)
	    If(l2.eq.0) then
	      Label1(icolumn)=0
	    else
	      call descen(LabelInt,l2,jfield)
	      Label1(icolumn)=LabelFin(jfield)
	    endif
	  enddo
	  if(Nfieldso.ne.0) then
	    Do Ifield=1,Nfieldso
	      LabelInt(Ifield)=Ifield
	    enddo
	  endif
	  Nfieldsi=Nfieldso
	enddo
	return
	end
	subroutine descen(label,labin,labout)
	integer*4 label(1)
	labout=labin
1	lab=label(labout)
	if(lab.eq.labout) return
	labout=lab
	goto 1
	end
