c.ps 60,68
c.hl 1 Radial Profile (Elliptical contour)
c.x SAIPH4
c.x Radial Profile> Elliptical contour
c.nf.ts 14,32
cA.Bijaoui	85/3	Nice Observatory
c.f
c.ap
c	Allows to determine the radial profile of an objet in elliptical rings.
c	That allows iterative determination to substract pixels for which the
cvalues are too far from the mean value for the corresponding distance to the
ccenter.
c	The threshold to eliminate data is the product of the mean standard
cdeviation with a named threshold parameter. A good value for this parameter
cis about 10.
c	The array LenA contains all the pixels corresponding
cto a given distance,
cand the array LenR only the pixels considered for the integration, after
citeration.
c	The array Sum1 contains the integrated profile, taking into account,
cif option, the eliminated pixels.
c.b.c; ^&Call and Parameters\&
c.b;CALL SAIPH4(ImageIn, Ncolumns, Nlines, Startx, Stepx, Starty, Stepy,
cXcenter, Ycenter, MajorAxis, MinorAxis, Orientation, Nprofil, Option,
cBackground, Profile, Status, LenA, LenR,
cSum1, Sum2, Sigma, Threshold)
c.b
c.nf.ts 14,27,30
cImageIn	R*4(Ncolumns,Nlines)	I	Input image
cNcolumns	I*4	I	Corresponding number of columns
cNlines	I*4	I	Corresponding number of lines
cStartX	R*4	I	Corresponding start x
cStepX	R*4	I	Corresponding step x
cStartY	R*4	I	Corresponding start y
cStepY	R*4	I	Corresponding y step
cXCenter	R*4	I	Physical abcissa of the integration center
cYcenter	R*4	I	Physical ordinate of the integration center
cMajorAxis	R*4	I	Ellipsis major axis
cMinorAxis	R*4	I	Ellipsis minor axis
cOrientation	R*4	I	Major axis orientation in degrees
cNprofil	I*4	I	Number of integration points
cOption	I*4	I	Working Option:
c	0		Simple radial profile without iteration
c	1		The same, with background determination
c			with the 5 last poinnts
c	2		1 iteration and background determination with
c			the 5 last points
cBackGround	R*4	I/O	Background value
cProfile	R*4(Nprofil)	O	Radial profile
cStatus	I*4	O	Error parameter:
c	0		No error
c	1		The integration area touch the edges
c	2		The integration center is out of the field
c			we don't integrate
cLenA	I*4(Nprofil)	O	Auxiliary array
cLenR	I*4(Nprofil)	O	Auxiliary array
cSum1	R*4(Nprofil)	O	Auxiliary array
cSum2	R*4(Nprofil)	O	Auxiliary array
cSigma	R*4	O	mean standard deviation
cThreshold	R*4	I	Threshold factor for rejection
c.f
	Subroutine SAIPH4(ImageIn, Ncolumns, Nlines, Startx, Stepx, Starty,
	1 Stepy, Xcenter, Ycenter, MajorAxis, MinorAxis, Orientation, Nprofil,
	2 Option, Background, Profile, Status, LenA, LenR, Sum1, Sum2,
	3 Sigma, Threshold)
	Integer*4 lenA(Nprofil), lenR(Nprofil), Option, Status
	Real*4 Sum1(Nprofil), Sum2(Nprofil), Profile(Nprofil)
	Real*4 MajorAxis, MinorAxis, ImageIn(Ncolumns,Nlines)
	Integer*4 Linecenter, ColumnCenter
	Elongation=MajorAxis/MinorAxis
	El=Elongation*Elongation
	Theta=Orientation*3.1415919/180.
	Ct=Cos(theta)
	St=Sin(theta)
	XXCenter=((Xcenter-Startx)/StepX)+1
	YYCenter=((Ycenter-Starty)/stepy)+1
	ColumnCenter=Ifix(xxcenter)
	LineCenter=Ifix(yycenter)
	If((ColumnCenter.lt.1).or.(ColumnCenter.gt.Ncolumns).or.
	1 (LineCenter.lt.1).or.(LineCenter.gt.Nlines)) then
	  Status=2
	  Return
	endif
	Nc1=ColumnCenter-Nprofil
	Nc2=ColumnCenter+nprofil+1
	Nl1=LineCenter-Nprofil
	Nl2=LineCenter+Nprofil+1
	If((nc1.lt.1).or.(nl1.lt.1).or.(nc2.gt.Ncolumns).or.(Nl2.gt.Nlines))
	1 status=1
	Nc1=Max0(1,Nc1)
	Nl1=Max0(1,Nl1)
	Nc2=Min0(Ncolumns,Nc2)
	Nl2=Min0(Nlines,Nl2)
	Iflag=0
 1000	CONTINUE
	Do Iprofil=1,Nprofil
	  LenA(Iprofil)=0
	  LenR(Iprofil)=0
	  Sum1(Iprofil)=0
	  Sum2(Iprofil)=0
	enddo
	Do iline=nl1,nl2
	  Y2=(float(iline)-yycenter)
	  Ys=Y2*st
	  Yc=Y2*ct
	  Do Icolumn=nc1,nc2
	    x2=(float(icolumn)-xxcenter)
	    Xs=X2*st
	    Xc=X2*ct
	    x2=(xc+ys)*(xc+ys)+el*(yc-xs)*(yc-xs)
 	    X2=SQRT(X2)
 	    Iprofil=IFIX(X2)+1
	    If(Iprofil.le.Nprofil) then
c	type *,Icolumn,Iline,ImageIn(Icolumn,Iline),Background
	      z=ImageIn(Icolumn,Iline)-Background
	      LenA(Iprofil)=LenA(Iprofil)+1
	      If(iflag.ne.0) then
		Q=ABS(z-Profile(Iprofil))/Threshold
		IF(Q.le.Sigma) then
		  lenR(Iprofil)=LenR(iprofil)+1
		  Sum1(Iprofil)=Sum1(Iprofil)+z
		  Sum2(Iprofil)=Sum2(Iprofil)+z*z
		endif
	      else
		  lenR(Iprofil)=LenR(iprofil)+1
		  Sum1(Iprofil)=Sum1(Iprofil)+z
		  Sum2(Iprofil)=Sum2(Iprofil)+z*z
	      endif
	    endif
	  enddo
	enddo
	Do Iprofil=1,Nprofil
	  If(LenR(Iprofil).ne.0) Profile(Iprofil)=Sum1(Iprofil)
	1 /Float(LenR(Iprofil))
	Enddo
	If(Option.ne.0) then
 	  NProfil4=NProfil-4
 	  S=0.
	  Do Iprofil=Nprofil4,Nprofil
	    S=S+Profile(Iprofil)
	  enddo
	  s5=s/5.
	  do Iprofil=1,Nprofil
	    Profile(Iprofil)=Profile(Iprofil)-s5
	    Sum1(Iprofil)=Sum1(Iprofil)-s5*LenR(Iprofil)
	    Sum2(Iprofil)=Sum2(Iprofil)-s5*s5*LenR(Iprofil)-2.*s5*Sum1(Iprofil)
	  enddo
	  Backround=Background+S5
	endif
	Nsigma=0
	Sigma=0.
	Do Iprofil=1,Nprofil
	  If(LenR(Iprofil).ne.0) then
	    s=(sum2(Iprofil)/float(lenr(iprofil))-Profile(Iprofil)**2)
	    If(s.gt.0.) then
	      sigma=sigma+sqrt(s)
	      Nsigma=Nsigma+1
	    endif
	  Endif
	enddo
	Sigma=Sigma/Float(Nsigma)
	Sum1(1)=Profile(1)*LenA(1)
	Do Iprofil=2,Nprofil
	  Sum1(Iprofil)=Sum1(Iprofil-1)+Profile(Iprofil)*LenA(Iprofil)
	Enddo
	If(Option.le.1) Return
	If(Iflag.eq.1) Return
	Iflag=1
	Goto 1000
	End
